@@ -21,10 +21,6 @@ from . import selections2 as sel2
 def readtime_dtype(basetype, names):
     """ Make a NumPy dtype appropriate for reading """
 
-    if basetype.kind == 'O':
-        # Special-dtype fields break indexing
-        basetype = numpy.dtype('O')
-
     if len(names) == 0:  # Not compound, or we want all fields
         return basetype
 
@@ -61,6 +57,13 @@ def make_new_dset(parent, shape=None, dtype=None, data=None,
         if data is not None and (numpy.product(shape) != numpy.product(data.shape)):
             raise ValueError("Shape tuple is incompatible with data")
 
+    tmp_shape = maxshape if maxshape is not None else shape
+    # Validate chunk shape
+    if isinstance(chunks, tuple) and (-numpy.array([ i>=j for i,j in zip(tmp_shape,chunks) if i is not None])).any():
+        errmsg = "Chunk shape must not be greater than data shape in any dimension. "\
+                 "{} is not compatible with {}".format(chunks, shape)
+        raise ValueError(errmsg)
+
     if isinstance(dtype, h5py.Datatype):
         # Named types are used as-is
         tid = dtype.id
@@ -324,8 +327,10 @@ class Dataset(HLObject):
         args = args if isinstance(args, tuple) else (args,)
 
         # Sort field indices from the rest of the args.
-        names = tuple(x for x in args if isinstance(x, str))
-        args = tuple(x for x in args if not isinstance(x, str))
+        names = tuple(x for x in args if isinstance(x, basestring))
+        args = tuple(x for x in args if not isinstance(x, basestring))
+        if not py3:
+            names = tuple(x.encode('utf-8') if isinstance(x, unicode) else x for x in names)
 
         def strip_fields(basetype):
             """ Strip extra dtype information from special types """
@@ -364,8 +369,9 @@ class Dataset(HLObject):
 
             return numpy.dtype([(name, basetype.fields[name][0]) for name in names])
 
-        if self._local.astype is not None:
-            new_dtype = readtime_dtype(self._local.astype, names)
+        new_dtype = getattr(self._local, 'astype', None)
+        if new_dtype is not None:
+            new_dtype = readtime_dtype(new_dtype, names)
         else:
             # This is necessary because in the case of array types, NumPy
             # discards the array information at the top level.
@@ -383,9 +389,9 @@ class Dataset(HLObject):
             sid = h5r.get_region(args[0], self.id)
             mshape = sel.guess_shape(sid)
             if mshape is None:
-                return np.array((0,), dtype=new_dtype)
+                return numpy.array((0,), dtype=new_dtype)
             if numpy.product(mshape) == 0:
-                return np.array(mshape, dtype=new_dtype)
+                return numpy.array(mshape, dtype=new_dtype)
             out = numpy.empty(mshape, dtype=new_dtype)
             sid_out = h5s.create_simple(mshape)
             sid_out.select_all()
@@ -457,12 +463,33 @@ class Dataset(HLObject):
         args = args if isinstance(args, tuple) else (args,)
 
         # Sort field indices from the slicing
-        names = tuple(x for x in args if isinstance(x, str))
-        args = tuple(x for x in args if not isinstance(x, str))
+        names = tuple(x for x in args if isinstance(x, basestring))
+        args = tuple(x for x in args if not isinstance(x, basestring))
+        if not py3:
+            names = tuple(x.encode('utf-8') if isinstance(x, unicode) else x for x in names)
 
         # Generally we try to avoid converting the arrays on the Python
         # side.  However, for compound literals this is unavoidable.
-        if self.dtype.kind == "O" or \
+        vlen = h5t.check_dtype(vlen=self.dtype)
+        if vlen not in (bytes, unicode, None):
+            try:
+                val = numpy.asarray(val, dtype=vlen)
+            except ValueError:
+                try:
+                    val = numpy.array([numpy.array(x, dtype=vlen)
+                                       for x in val], dtype=self.dtype)
+                except ValueError:
+                    pass
+            if vlen == val.dtype:
+                if val.ndim > 1:
+                    tmp = numpy.empty(shape=val.shape[:-1], dtype=object)
+                    tmp.ravel()[:] = [i for i in val.reshape(
+                        (numpy.product(val.shape[:-1]), val.shape[-1]))]
+                else:
+                    tmp = numpy.array([None], dtype=object)
+                    tmp[0] = val
+                val = tmp
+        elif self.dtype.kind == "O" or \
           (self.dtype.kind == 'V' and \
           (not isinstance(val, numpy.ndarray) or val.dtype.kind != 'V') and \
           (self.dtype.subdtype == None)):
