@@ -16,6 +16,11 @@
 #ifndef H5PY_COMPAT
 #define H5PY_COMPAT
 
+#if (MPI_VERSION < 3) && !defined(PyMPI_HAVE_MPI_Message)
+typedef void *PyMPI_MPI_Message;
+#define MPI_Message PyMPI_MPI_Message
+#endif
+
 #include <stddef.h>
 #include "Python.h"
 #include "numpy/arrayobject.h"
